import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/pink_floyd1242.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _ozzy_osbourne334();
}

class _ozzy_osbourne334 extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _himsa840;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final evanescence841 = def_leppard335.job_for_a_cowboy21(
      context,
      necrophagist1088: true,
    );
    if (evanescence841 != null) {
      _himsa840 = widget.maskingConfig.merge(
        evanescence841.the_beatles171,
      );
    } else {
      _himsa840 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return def_leppard335(
      the_beatles171: _himsa840,
      nevermore222: widget.child,
    );
  }
}


class def_leppard335 extends InheritedWidget {
  
  final MaskingConfig the_beatles171;

  
  const def_leppard335({
    required this.the_beatles171,
    Key? damageplan977,
    required Widget nevermore222,
  }) : super(key: damageplan977, child: nevermore222);

  
  static def_leppard335? job_for_a_cowboy21(
    BuildContext poison1087, {
    required bool necrophagist1088,
  }) {
    return necrophagist1088
        ? poison1087.dependOnInheritedWidgetOfExactType<def_leppard335>()
        : poison1087.motorhead588<def_leppard335>();
  }

  @override
  bool updateShouldNotify(def_leppard335 oldWidget) {
    return oldWidget.the_beatles171 != the_beatles171;
  }
}

extension on BuildContext {
  
  T? motorhead588<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<def_leppard335>()
        ?.widget as T?;
  }
}
