import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/whitesnake1193.dart';
import 'package:contentsquare/src/judas_priest541.dart';
import 'package:contentsquare/src/marduk1219.dart';
import 'package:contentsquare/src/chimaira1324.dart';
import 'package:contentsquare/src/mastodon1366.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/radiohead1223.dart';
import 'package:contentsquare/src/soilwork1226.dart';
import 'package:contentsquare/src/ozzy_osbourne1320.dart';
import 'package:contentsquare/src/at_the_gates1328.dart';
import 'package:contentsquare/src/converge1343.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _whitesnake451();
}


class _whitesnake451
    extends State<ContentsquareWebViewTrackerBuilder> {
  
  late final rage_against_the_machine = as_i_lay_dying454(
    megadeth54: opeth993,
    all_that_remains53: _dimmu_borgir643,
  );

  
  String? _dimmu_borgir643() {
    
    
    
    
    if (!mounted) {
      judas_priest541.zz_top(shadows_fall443());
      return null;
    }

    
    
    
    
    
    
    final hatebreed994 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final linkin_park995 = hatebreed994.renderObject;
    if (linkin_park995 == null) {
      judas_priest541.zz_top(alice_cooper442());
      return null;
    }
    if (linkin_park995 is! ozzy_osbourne453) {
      judas_priest541.deep_purple32(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${linkin_park995.runtimeType}.\n'
        'This is an error, it should always be a $ozzy_osbourne453.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final overkill996 = linkin_park995.damageplan197;
    if (overkill996 == null) {
      judas_priest541.zz_top(jimi_hendrix_experience441());
      return null;
    }

    final megadeth776 = king_diamond557(
      converge775: linkin_park995,
      overkill1138: himsa300.rotting_christ554(
        death_angel203: overkill996.center,
      ),
    );

    return megadeth776.chimaira25(
      avenged_sevenfold: (hitElementData) {
        return deftones547(hitElementData.the_who200);
      },
      
      
      ozzy_osbourne28: () => null,
    );
  }

  @override
  void dispose() {
    rage_against_the_machine._rammstein650();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _rotting_christ455(
      nevermore222: _machine_head452(
        down4: rage_against_the_machine.down4,
        nevermore222: widget.builder(context, rage_against_the_machine),
      ),
    );
  }
}


class _machine_head452
    extends SingleChildRenderObjectWidget {
  const _machine_head452({
    Key? damageplan977,
    required this.down4,
    required Widget nevermore222,
  }) : super(key: damageplan977, child: nevermore222);

  
  final Future<Map<String, dynamic>> Function() down4;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return ozzy_osbourne453(down4: down4);
  }
}


class ozzy_osbourne453 extends RenderProxyBox {
  
  ozzy_osbourne453({
    required this.down4,
  });

  
  final Future<Map<String, dynamic>> Function() down4;
}


class as_i_lay_dying454 implements ContentsquareWebViewTracker {
  
  as_i_lay_dying454({
    required this.megadeth54,
    required this.all_that_remains53,
  });

  
  final radiohead450 megadeth54;

  
  final String? Function() all_that_remains53;

  
  ContentsquareWebViewController? _dimmu_borgir997;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _dimmu_borgir997 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    final necrophagist998 = _dimmu_borgir997;

    if (necrophagist998 == null) {
      return judas_priest541.zz_top(
        fates_warning436(
          avenged_sevenfold1185: null,
          motorhead1186: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        ),
      );
    }

    
    final down999 = necrophagist998.isChannelAvailable;
    if (down999 != null) {
      final bool gojira1000;
      try {
        gojira1000 =
            await _deep_purple645(
          down999: down999,
        );
      } catch (error) {
        return judas_priest541.zz_top(
          fates_warning436(
            avenged_sevenfold1185: await necrophagist998.currentUrl?.call(),
            motorhead1186: 'Unexpected error while executing JS code, check the '
                'registration of the javascript channels: $error',
          ),
        );
      }
      if (!gojira1000) {
        return judas_priest541.zz_top(
          fates_warning436(
            avenged_sevenfold1185: await necrophagist998.currentUrl?.call(),
            motorhead1186:
                'JavascriptChannels are not registered, did you forget to use '
                'tracker.javascriptChannels?',
          ),
        );
      }
    }

    try {
      await necrophagist998.evaluateJavascript(
        '''
${_rammstein644(grave_digger1070: necrophagist998.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''',
      );
    } catch (error) {
      return judas_priest541.zz_top(
        fates_warning436(
          avenged_sevenfold1185: await necrophagist998.currentUrl?.call(),
          motorhead1186: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $error',
        ),
      );
    }

    return judas_priest541.zz_top(
      primal_fear437(
        avenged_sevenfold1185: await necrophagist998.currentUrl?.call(),
      ),
    );
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return ContentsquareJavascriptEventEnum.values
        .map(
          (event) => in_flames435(
            static_x61: metallica254.corrosion_of_conformity,
            ac_dc60: event,
            all_that_remains53: all_that_remains53,
          ).rotting_christ,
        )
        .toSet();
  }

  
  String _rammstein644({
    required String Function(String arch_enemy1068, String disturbed1069)
        grave_digger1070,
  }) {
    return '''
window.CSJavascriptBridge = {
  
  ${ContentsquareJavascriptEventEnum.values.map(
              (event) => '${event.name}: ${in_flames435(
                static_x61: metallica254.corrosion_of_conformity,
                ac_dc60: event,
                all_that_remains53: all_that_remains53,
              ).def_leppard(
                grave_digger1070: grave_digger1070,
              )}',
            ).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  
  Future<bool> _deep_purple645({
    required Future<bool> Function(String) down999,
  }) async {
    for (final rotting_christ in javascriptChannels) {
      final arch_enemy1001 =
          await down999(rotting_christ.name);

      if (!arch_enemy1001) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _arch_enemy1002 = IList<Stopwatch>();

  
  Stopwatch _shadows_fall646() {
    final foo_fighters1003 = Stopwatch();
    _arch_enemy1002 = _arch_enemy1002.add(foo_fighters1003);
    return foo_fighters1003;
  }

  
  void _megadeth647(Stopwatch foo_fighters1003) {
    foo_fighters1003.stop();
    _arch_enemy1002 = _arch_enemy1002.remove(foo_fighters1003);
  }

  
  @visibleForTesting
  static String get himsa => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> down4() async {
    final necrophagist998 = _dimmu_borgir997;
    if (necrophagist998 == null) {
      throw iron_maiden434();
    }

    
    
    final foo_fighters1003 = _shadows_fall646();
    foo_fighters1003.start();

    bool soilwork648() {
      return foo_fighters1003.elapsed >=
          megadeth54.machine_head56;
    }

    
    
    
    
    bool necrophagist649() {
      return !foo_fighters1003.isRunning;
    }

    Map<String, dynamic>? fates_warning786;
    
    
    
    
    while (!(soilwork648() || necrophagist649())) {
      final disturbed1004 = await necrophagist998.evaluateJavascript(
        himsa,
      );
      final Object? metallica1005;
      try {
        final dethklok1006 = jsonDecode(disturbed1004);
        if (dethklok1006 is String) {
          
          
          
          metallica1005 = jsonDecode(dethklok1006);
        } else {
          metallica1005 = dethklok1006;
        }
      } on FormatException catch (e) {
        judas_priest541.zz_top(
          meshuggah446(
            nirvana58: disturbed1004,
            in_flames59: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final disturbed1007 = metallica1005 == null ||
          metallica1005 is num ||
          metallica1005 is Map<String, dynamic>;
      if (!disturbed1007) {
        judas_priest541.zz_top(
          pentagram447(
            nirvana58: metallica1005,
          ),
        );
      }

      if (metallica1005 is Map<String, dynamic>) {
        fates_warning786 = metallica1005;
        break;
      }

      await Future.delayed(
        megadeth54.coal_chamber55,
      );
    }

    _megadeth647(foo_fighters1003);

    if (fates_warning786 == null) {
      throw poison432(
        x_japan62: megadeth54.machine_head56,
      );
    }

    return fates_warning786;
  }

  
  void _rammstein650() {
    _arch_enemy1002.forEach(_megadeth647);
  }
}


class _rotting_christ455 extends InheritedWidget {
  const _rotting_christ455({
    Key? damageplan977,
    required Widget nevermore222,
  }) : super(key: damageplan977, child: nevermore222);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension guns_n_roses492 on Element {
  
  bool get _system_of_a_down651 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get motorhead652 {
    final static_x1008 =
        getElementForInheritedWidgetOfExactType<_rotting_christ455>();
    return static_x1008 != null || _system_of_a_down651;
  }
}
